/**
 * $Id$ 
 * 
 * Copyright (c) 2008, Jens Elkner.
 * All rights reserved.
 *
 * Code licensed under the BSD License: http://developer.yahoo.net/yui/license.txt
 */
package com.yahoo.platform.yui.compressor;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;

import org.apache.tools.ant.types.Parameter;

/**
 * A CSS reader, which compresses the input per default.
 * NOTE: The reader always reads in the complete stream as soon as a 
 * {@link #read(char[], int, int)} or {@link #skip(long)} or {@link #mark(int)}
 * is invoked on this filter and than buffers the result for consecutive 
 * operations. So you may need to invoke ant with something like {@code -Xmx256m}
 * for large files.
 * <p>
 * Dueto the bad design of {@link org.apache.tools.ant.filters.BaseFilterReader} 
 * this filter is not able to use the ant logging system and thus emits all
 * error messages to stderr, warnings and verbose output to stdout.
 * 
 * @author Jens Elkner
 * @version $Revision$
 */
public class CssFilter
	extends YUIFilter
{

	/**
	 * Create a new CSS compressing Filter, which breaks the resulting String 
	 * into lines of max. {@var linebreak} columns.
	 * @param in			the reader, which provides the CSS to compress.
	 * @param linebreak		max. length of resulting lines. A negative value 
	 * 			implies unlimited length.
	 */
	public CssFilter(Reader in, int linebreak) {
		super(in, linebreak);
	}

	/**
	 * Create a new CSS compressing Filter.
	 * @param in			the reader, which provides the CSS to compress.
	 */
	public CssFilter(Reader in) {
		this(in, -1);
	}

	/**
	 * Creates an YUI CSS compressor, parses the input stream and 
	 * provides the results in the returned reader.
	 */
	@Override
	protected StringReader getResults() throws IOException {
		CssCompressor compressor = new CssCompressor(in);
		StringWriter out = new StringWriter(4096);
		compressor.compress(out, linebreak);
		return new StringReader(out.toString());
	}

	/**
	 * Additional parameters for this filter.
	 * @see YUIFilter#setParameters(Parameter[])
	 */
	@Override
	public void setParameters(Parameter[] parameters) {
		if (parameters == null) {
			return;
		}
		ArrayList<Parameter> params = setParams(parameters);
		if (initialized) {
			return;
		}
		for (Parameter param : params) {
			System.err.println("Unknown parameter '" + param.getName() + "' ignored");
		}
	}
}
